"""
ORPL
================
the Open Raman Processing Library is a collection of tools for the processing, analysis and
visualization of Raman spectrum.
"""

from orpl import (
    baseline_removal,
    calibration,
    cosmic_ray,
    datatypes,
    file_io,
    metrics,
    normalization,
    plot,
    synthetic,
)

ORPL_VERSION = "1.0.3"
